@@ -1,6 +1,7 @@
 
 # global variables
 the <- new.env(parent = emptyenv())
+the$paths <- new.env(parent = emptyenv())
 
 # detect if we're running on CI
 ci <- function() {
@@ -18,6 +19,11 @@ building <- function() {
     grepl("Rbuild", basename(dirname(getwd())), fixed = TRUE)
 }
 
+# detect if we're running within R CMD INSTALL
+installing <- function() {
+  nzchar(Sys.getenv("R_INSTALL_PKG"))
+}
+
 # are we running code within R CMD check?
 checking <- function() {
   "CheckExEnv" %in% search() ||
