@@ -127,9 +127,10 @@ install <- function(packages = NULL,
   renv_scope_libpaths(libpaths)
 
   # check for explicitly-provided type -- we handle this specially for PPM
-  type <- type %||% getOption("pkgType")
-  renv_scope_binding(the, "install_pkg_type", type)
-  renv_scope_options(pkgType = type)
+  if (!is.null(type)) {
+    renv_scope_binding(the, "install_pkg_type", type)
+    renv_scope_options(pkgType = type)
+  }
 
   # override repositories if requested
   repos <- repos %||% config$repos.override()
