@@ -137,3 +137,27 @@ test_that("pak's cran remotes are considered cranlike", {
   expect_identical(!!actual, !!expected)
 
 })
+
+test_that("we format github remotes appropriately", {
+
+  record <- list(
+    Package        = "skeleton",
+    Version        = "1.1.0",
+    RemoteType     = "github",
+    RemoteUsername = "kevinushey",
+    RemoteRepo     = "skeleton",
+    RemoteRef      = "main",
+    RemoteSha      = "e4aafb92b86ba7eba3b7036d9d96fdfb6c32761a",
+    RemoteSubdir   = "subdir"
+  )
+
+  remote <- renv_record_format_remote(record, compact = FALSE)
+  expect_equal(remote, "kevinushey/skeleton:subdir@e4aafb92b86ba7eba3b7036d9d96fdfb6c32761a")
+
+  remote <- renv_record_format_remote(record, compact = TRUE)
+  expect_equal(remote, "kevinushey/skeleton:subdir")
+
+  remote <- renv_record_format_remote(record, pak = TRUE)
+  expect_equal(remote, "kevinushey/skeleton/subdir@e4aafb92b86ba7eba3b7036d9d96fdfb6c32761a")
+
+})
